const express = require('express');

const schoolApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

let rqf = new ReqQueryFields();
let rqfCount = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addValue({
    name: 'id',
    table: 'escola',
    tableField: 'id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    }
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'escola'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: 'escola'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'year',
    table: 'escola',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
        table: 'escola'
    }
});

rqfCount.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'id',
    table: 'escola',
    tableField: 'id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    }
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'escola'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: 'escola'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: 'nome',
    resultField: 'region_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'regiao_id',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'year',
    table: 'escola',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
        table: 'escola'
    }
}).addValue({
    name: 'adm_dependency',
    table: 'dependencia_adm',
    tableField: 'nome',
    resultField: 'adm_dependency_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'dependencia_adm_id',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'adm_dependency_detailed',
    table: 'dependencia_adm',
    tableField: 'nome',
    resultField: 'adm_dependency_detailed_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'dependencia_adm_priv',
        foreignTable: 'escola'
    }
}).addValue({
    name: 'location',
    table: 'escola',
    tableField: 'cod_localizacao',
    resultField: 'cod_localizacao',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_localizacao'
    }
}).addValue({
    name: 'cook_room',
    table: 'escola',
    tableField: 'cozinha',
    resultField: 'cook_room_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'cozinha'
    }
}).addValue({
    name: 'government_agreement',
    table: 'escola',
    tableField: 'conveniada_pp',
    resultField: 'government_agreement_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'conveniada_pp'
    }
}).addValue({
    name: 'informatics_lab',
    table: 'escola',
    tableField: 'lab_informatica',
    resultField: 'informatics_lab_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'lab_informatica'
    }
}).addValue({
    name: 'science_lab',
    table: 'escola',
    tableField: 'lab_ciencias',
    resultField: 'informatics_lab_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'lab_ciencias'
    }
}).addValue({
    name: 'special_attendence_room',
    table: 'escola',
    tableField: 'sala_atendimento_especial',
    resultField: 'special_attendence_room_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'sala_atendimento_especial'
    }
}).addValue({
    name: 'indor_sports_court',
    table: 'escola',
    tableField: 'quadra_esportes_coberta',
    resultField: 'indor_sports_court_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'quadra_esportes_coberta'
    }
}).addValue({
    name: 'education_eja',
    table: 'escola',
    tableField: 'ensino_eja',
    resultField: 'education_eja_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'ensino_eja'
    }
}).addValue({
    name: 'education_professional',
    table: 'escola',
    tableField: 'educacao_profissional',
    resultField: 'education_professional_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'educacao_profissional'
    }
}).addValue({
    name: 'education_middle_school',
    table: 'escola',
    tableField: 'reg_medio_medio',
    resultField: 'education_middle_school_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'reg_medio_medio'
    }
}).addValue({
    name: 'education_end_elementary_school',
    table: 'escola',
    tableField: 'reg_fund_af',
    resultField: 'education_end_elementary_school_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'reg_fund_af'
    }
}).addValue({
    name: 'education_begin_elementary_school',
    table: 'escola',
    tableField: 'reg_fund_ai',
    resultField: 'education_begin_elementary_school_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'reg_fund_ai'
    }
}).addValue({
    name: 'education_preschool_child',
    table: 'escola',
    tableField: 'reg_infantil_preescola',
    resultField: 'education_preschool_child_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'reg_infantil_preescola'
    }
}).addValue({
    name: 'directors_room',
    table: 'escola',
    tableField: 'sala_diretoria',
    resultField: 'directors_room_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'sala_diretoria'
    }
}).addValue({
    name: 'teacher_room',
    table: 'escola',
    tableField: 'sala_professor',
    resultField: 'teacher_room_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'sala_professor'
    }
}).addValue({
    name: 'playground',
    table: 'escola',
    tableField: 'parque_infantil',
    resultField: 'playground_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'parque_infantil'
    }
}).addValue({
    name: 'nusery',
    table: 'escola',
    tableField: 'bercario',
    resultField: 'nusery_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'bercario'
    }
}).addValue({
    name: 'toilet_inside_building',
    table: 'escola',
    tableField: 'sanitario_dentro_predio',
    resultField: 'toilet_inside_building_name',
    where: {
        relation: '=',
        type: 'boolean',
        field: 'sanitario_dentro_predio'
    }
});

schoolApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    console.log(req.filter);
    if(typeof req.filter === 'undefined' || Object.keys(req.filter).length === 0) {
        res.status(400);
        next({
            status: 400,
            message: 'Wrong/No filter specified'
        });
    }
    req.sql.from('escola')
        .field('escola.id')
        .field('escola.ano_censo', 'year')
        .field('escola.nome_escola', 'name')
        .field('escola.estado_id', 'state_id')
        .field('escola.municipio_id', 'city_id');
    next();
}, query, response('school'));

schoolApp.get('/count', rqfCount.parse(), rqfCount.build(), (req, res, next) => {

    req.sql.from('escola')
        .field('COUNT(escola.id)', 'total')
        .field("'Brasil'", 'name')
        .field('escola.ano_censo', 'year')
        .group('escola.ano_censo')
        .order('escola.ano_censo')
        .where('escola.situacao_de_funcionamento = 1 AND escola.ensino_regular = 1');
    next();
}, query, response('school'));

module.exports = schoolApp;
